@@ -623,7 +623,7 @@ class MultiModalClassificationModel:
             if args.save_model_every_epoch:
                 self.save_model(output_dir_current, model=model)
 
-            if args.evaluate_during_training:
+            if args.evaluate_during_training and args.evaluate_each_epoch:
                 results, _ = self.eval_model(
                     eval_data,
                     files_list=files_list,
@@ -819,6 +819,12 @@ class MultiModalClassificationModel:
         out_label_ids = None
         model.eval()
 
+        if args.n_gpu > 1:
+            model = torch.nn.DataParallel(model)
+
+        if args.fp16:
+            from torch.cuda import amp
+
         for batch in tqdm(eval_dataloader, disable=args.silent or silent, desc="Running Evaluation"):
             batch = tuple(t.to(device) for t in batch)
             labels = batch[5]
@@ -829,7 +835,9 @@ class MultiModalClassificationModel:
                 logits = outputs[0]  # Different from default behaviour
                 tmp_eval_loss = self.criterion(logits, labels)
 
-                eval_loss += tmp_eval_loss.mean().item()
+                if self.args.n_gpu > 1:
+                    tmp_eval_loss = tmp_eval_loss.mean()
+                eval_loss += tmp_eval_loss.item()
 
             nb_eval_steps += 1
 
@@ -1018,17 +1026,29 @@ class MultiModalClassificationModel:
         preds = None
         out_label_ids = None
 
+        if args.n_gpu > 1:
+            model = torch.nn.DataParallel(model)
+
+        if args.fp16:
+            from torch.cuda import amp
+
         for batch in tqdm(eval_dataloader, disable=args.silent, desc="Running Prediction"):
             batch = tuple(t.to(device) for t in batch)
             labels = batch[5]
             with torch.no_grad():
                 inputs = self._get_inputs_dict(batch)
-
-                outputs = model(**inputs)
-                logits = outputs[0]  # Different from default behaviour
+                if self.args.fp16:
+                    with amp.autocast():
+                        outputs = model(**inputs)
+                        logits = outputs[0]  # Different from default behaviour
+                else:
+                    outputs = model(**inputs)
+                    logits = outputs[0]  # Different from default behaviour
                 tmp_eval_loss = self.criterion(logits, labels)
 
-                eval_loss += tmp_eval_loss.mean().item()
+                if self.args.n_gpu > 1:
+                    tmp_eval_loss = tmp_eval_loss.mean()
+                eval_loss += tmp_eval_loss.item()
 
             nb_eval_steps += 1
 
