@@ -41,6 +41,7 @@ class ModelArgs:
     evaluate_during_training_silent: bool = True
     evaluate_during_training_steps: int = 2000
     evaluate_during_training_verbose: bool = False
+    evaluate_each_epoch: bool = True
     fp16: bool = True
     gradient_accumulation_steps: int = 1
     learning_rate: float = 4e-5
@@ -118,6 +119,7 @@ class ClassificationArgs(ModelArgs):
     lazy_text_a_column: bool = None
     lazy_text_b_column: bool = None
     lazy_text_column: int = 0
+    onnx: bool = False
     regression: bool = False
     sliding_window: bool = False
     stride: float = 0.8
@@ -151,6 +153,7 @@ class NERArgs(ModelArgs):
     labels_list: list = field(default_factory=list)
     lazy_loading: bool = False
     lazy_loading_start_line: int = 0
+    onnx: bool = False
 
 
 @dataclass
