@@ -25,10 +25,13 @@ from transformers.modeling_electra import (
     ElectraModel,
     ElectraPreTrainedModel,
 )
-from transformers.modeling_roberta import ROBERTA_PRETRAINED_MODEL_ARCHIVE_LIST, RobertaClassificationHead
+from transformers.modeling_roberta import (
+    ROBERTA_PRETRAINED_MODEL_ARCHIVE_LIST,
+    RobertaClassificationHead,
+    RobertaForQuestionAnswering,
+)
 from transformers.modeling_utils import PreTrainedModel, SequenceSummary
 from transformers.modeling_xlm_roberta import XLM_ROBERTA_PRETRAINED_MODEL_ARCHIVE_LIST
-from transformers.modeling_roberta import RobertaForQuestionAnswering
 
 
 class BertForMultiLabelSequenceClassification(BertPreTrainedModel):
