@@ -146,7 +146,18 @@ class QuestionAnsweringModel:
 
         config_class, model_class, tokenizer_class = MODEL_CLASSES[model_type]
         self.config = config_class.from_pretrained(model_name, **self.args.config)
-        self.model = model_class.from_pretrained(model_name, config=self.config, **kwargs)
+        if not self.args.quantized_model:
+            self.model = model_class.from_pretrained(model_name, config=self.config, **kwargs)
+        else:
+            quantized_weights = torch.load(os.path.join(model_name, "pytorch_model.bin"))
+            self.model = model_class.from_pretrained(None, config=self.config, state_dict=quantized_weights)
+
+        if self.args.dynamic_quantize:
+            self.model = torch.quantization.quantize_dynamic(self.model, {torch.nn.Linear}, dtype=torch.qint8)
+        if self.args.quantized_model:
+            self.model.load_state_dict(quantized_weights)
+        if self.args.dynamic_quantize:
+            self.args.quantized_model = True
 
         if use_cuda:
             if torch.cuda.is_available():
@@ -164,6 +175,12 @@ class QuestionAnsweringModel:
 
         self.results = {}
 
+        if self.args.fp16:
+            try:
+                from torch.cuda import amp
+            except AttributeError:
+                raise AttributeError("fp16 requires Pytorch >= 1.6. Please update Pytorch or turn off fp16.")
+
         self.tokenizer = tokenizer_class.from_pretrained(model_name, do_lower_case=self.args.do_lower_case, **kwargs)
 
         self.args.model_name = model_name
@@ -308,7 +325,7 @@ class QuestionAnsweringModel:
             train_dataset, output_dir, show_running_loss=show_running_loss, eval_data=eval_data, **kwargs
         )
 
-        self._save_model(model=self.model)
+        self.save_model(model=self.model)
 
         logger.info(" Training of {} model complete. Saved to {}.".format(self.args.model_type, output_dir))
 
@@ -524,7 +541,7 @@ class QuestionAnsweringModel:
                         # Save model checkpoint
                         output_dir_current = os.path.join(output_dir, "checkpoint-{}".format(global_step))
 
-                        self._save_model(output_dir_current, optimizer, scheduler, model=model)
+                        self.save_model(output_dir_current, optimizer, scheduler, model=model)
 
                     if args.evaluate_during_training and (
                         args.evaluate_during_training_steps > 0
@@ -538,7 +555,7 @@ class QuestionAnsweringModel:
                         output_dir_current = os.path.join(output_dir, "checkpoint-{}".format(global_step))
 
                         if args.save_eval_checkpoints:
-                            self._save_model(output_dir_current, optimizer, scheduler, model=model, results=results)
+                            self.save_model(output_dir_current, optimizer, scheduler, model=model, results=results)
 
                         training_progress_scores["global_step"].append(global_step)
                         training_progress_scores["train_loss"].append(current_loss)
@@ -554,11 +571,11 @@ class QuestionAnsweringModel:
 
                         if not best_eval_metric:
                             best_eval_metric = results[args.early_stopping_metric]
-                            self._save_model(args.best_model_dir, optimizer, scheduler, model=model, results=results)
+                            self.save_model(args.best_model_dir, optimizer, scheduler, model=model, results=results)
                         if best_eval_metric and args.early_stopping_metric_minimize:
                             if results[args.early_stopping_metric] - best_eval_metric < args.early_stopping_delta:
                                 best_eval_metric = results[args.early_stopping_metric]
-                                self._save_model(
+                                self.save_model(
                                     args.best_model_dir, optimizer, scheduler, model=model, results=results
                                 )
                                 early_stopping_counter = 0
@@ -579,7 +596,7 @@ class QuestionAnsweringModel:
                         else:
                             if results[args.early_stopping_metric] - best_eval_metric > args.early_stopping_delta:
                                 best_eval_metric = results[args.early_stopping_metric]
-                                self._save_model(
+                                self.save_model(
                                     args.best_model_dir, optimizer, scheduler, model=model, results=results
                                 )
                                 early_stopping_counter = 0
@@ -605,12 +622,12 @@ class QuestionAnsweringModel:
                 os.makedirs(output_dir_current, exist_ok=True)
 
             if args.save_model_every_epoch:
-                self._save_model(output_dir_current, optimizer, scheduler, model=model)
+                self.save_model(output_dir_current, optimizer, scheduler, model=model)
 
-            if args.evaluate_during_training:
+            if args.evaluate_during_training and args.evaluate_each_epoch:
                 results, _ = self.eval_model(eval_data, verbose=False, **kwargs)
 
-                self._save_model(output_dir_current, optimizer, scheduler, results=results)
+                self.save_model(output_dir_current, optimizer, scheduler, results=results)
 
                 training_progress_scores["global_step"].append(global_step)
                 training_progress_scores["train_loss"].append(current_loss)
@@ -624,11 +641,11 @@ class QuestionAnsweringModel:
 
                 if not best_eval_metric:
                     best_eval_metric = results[args.early_stopping_metric]
-                    self._save_model(args.best_model_dir, optimizer, scheduler, model=model, results=results)
+                    self.save_model(args.best_model_dir, optimizer, scheduler, model=model, results=results)
                 if best_eval_metric and args.early_stopping_metric_minimize:
                     if results[args.early_stopping_metric] - best_eval_metric < args.early_stopping_delta:
                         best_eval_metric = results[args.early_stopping_metric]
-                        self._save_model(args.best_model_dir, optimizer, scheduler, model=model, results=results)
+                        self.save_model(args.best_model_dir, optimizer, scheduler, model=model, results=results)
                         early_stopping_counter = 0
                     else:
                         if args.use_early_stopping and args.early_stopping_consider_epochs:
@@ -647,7 +664,7 @@ class QuestionAnsweringModel:
                 else:
                     if results[args.early_stopping_metric] - best_eval_metric > args.early_stopping_delta:
                         best_eval_metric = results[args.early_stopping_metric]
-                        self._save_model(args.best_model_dir, optimizer, scheduler, model=model, results=results)
+                        self.save_model(args.best_model_dir, optimizer, scheduler, model=model, results=results)
                         early_stopping_counter = 0
                     else:
                         if args.use_early_stopping and args.early_stopping_consider_epochs:
@@ -736,6 +753,12 @@ class QuestionAnsweringModel:
         nb_eval_steps = 0
         model.eval()
 
+        if args.n_gpu > 1:
+            model = torch.nn.DataParallel(model)
+
+        if self.args.fp16:
+            from torch.cuda import amp
+
         all_results = []
         for batch in tqdm(eval_dataloader, disable=args.silent, desc="Running Evaluation"):
             batch = tuple(t.to(device) for t in batch)
@@ -763,8 +786,13 @@ class QuestionAnsweringModel:
                 if args.model_type in ["xlnet", "xlm"]:
                     inputs.update({"cls_index": batch[4], "p_mask": batch[5]})
 
-                outputs = model(**inputs)
-                eval_loss += outputs[0].mean().item()
+                if self.args.fp16:
+                    with amp.autocast():
+                        outputs = model(**inputs)
+                        eval_loss += outputs[0].mean().item()
+                else:
+                    outputs = model(**inputs)
+                    eval_loss += outputs[0].mean().item()
 
                 for i, example_index in enumerate(example_indices):
                     eval_feature = features[example_index.item()]
@@ -875,6 +903,12 @@ class QuestionAnsweringModel:
 
         model.eval()
 
+        if args.n_gpu > 1:
+            model = torch.nn.DataParallel(model)
+
+        if self.args.fp16:
+            from torch.cuda import amp
+
         all_results = []
         for batch in tqdm(eval_dataloader, disable=args.silent, desc="Running Prediction"):
             batch = tuple(t.to(device) for t in batch)
@@ -902,7 +936,11 @@ class QuestionAnsweringModel:
                 if args.model_type in ["xlnet", "xlm"]:
                     inputs.update({"cls_index": batch[4], "p_mask": batch[5]})
 
-                outputs = model(**inputs)
+                if self.args.fp16:
+                    with amp.autocast():
+                        outputs = model(**inputs)
+                else:
+                    outputs = model(**inputs)
 
                 for i, example_index in enumerate(example_indices):
                     eval_feature = features[example_index.item()]
@@ -1040,7 +1078,7 @@ class QuestionAnsweringModel:
 
         return training_progress_scores
 
-    def _save_model(self, output_dir=None, optimizer=None, scheduler=None, model=None, results=None):
+    def save_model(self, output_dir=None, optimizer=None, scheduler=None, model=None, results=None):
         if not output_dir:
             output_dir = self.args.output_dir
         os.makedirs(output_dir, exist_ok=True)
@@ -1054,7 +1092,7 @@ class QuestionAnsweringModel:
             if optimizer and scheduler and self.args.save_optimizer_and_scheduler:
                 torch.save(optimizer.state_dict(), os.path.join(output_dir, "optimizer.pt"))
                 torch.save(scheduler.state_dict(), os.path.join(output_dir, "scheduler.pt"))
-            self._save_model_args(output_dir)
+            self.save_model_args(output_dir)
 
         if results:
             output_eval_file = os.path.join(output_dir, "eval_results.txt")
@@ -1062,7 +1100,7 @@ class QuestionAnsweringModel:
                 for key in sorted(results.keys()):
                     writer.write("{} = {}\n".format(key, str(results[key])))
 
-    def _save_model_args(self, output_dir):
+    def save_model_args(self, output_dir):
         os.makedirs(output_dir, exist_ok=True)
         self.args.save(output_dir)
 
